Package: gpciIntCensor
Type: Package
Title: Generalized Process Capability Indices for Interval-Censored
        Data
Version: 0.1.0
Authors@R: c(
    person("Shikhar", "Tyagi",
           email = "shikhar1093tyagi@gmail.com",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-1606-0844")),
    person("Sumit", "Kumar",
           email = "stats.sumitbhal@gmail.com",
           role = "aut"),
    person("Arvind", "Pandey",
           email = "arvindmzu@gmail.com",
           role = "aut"),
    person("Bhupendra", "Singh",
           email = "bhupendra.rana@gmail.com",
           role = "aut"),
    person("Vrijesh", "Tripathi",
           email = "vrijesh.tripathi@uwi.edu",
           role = "aut")
    )
Description: A comprehensive framework for computing, estimating, and validating 
    Generalized Process Capability Indices (GPCIs) under interval-censored data. 
    Supports user-supplied probability density functions (PDF/PMF), cumulative 
    distribution functions (CDF), and survival functions (SF). Parameter estimation 
    is performed using Maximum Likelihood Estimation for interval-censored data via 
    the MleCensoR package. Computes classical and generalized capability indices 
    including Cpy (Maiti et al., 2010), Spmk (Dey & Saha, 2019), CpTk (Saha et al., 
    2019), Cpc (Saha et al., 2022), CNpmc (Alotaibi et al., 2022), CNpmkc (Saha et 
    al., 2024), CNpk (Saha et al., 2018), and Vannman's Cp(u,v) family. Provides 
    parametric and non-parametric bootstrap confidence intervals at 90%, 95%, and 99% 
    confidence levels using percentile, normal, basic, BCa, BCp, and studentized 
    bootstrap methods. Computes standard errors, mean squared errors, and coverage 
    probabilities for both distribution parameters and capability indices.
    References:
    Maiti, Saha & Nanda (2010) <doi:10.1080/16843703.2010.11673233>,
    Saha, Dey & Maiti (2018) <doi:10.1080/21681015.2018.1437793>,
    Dey & Saha (2019) <doi:10.1007/s41872-019-00081-4>,
    Saha, Dey & Maiti (2019) <doi:10.1007/s13198-019-00789-7>,
    Alotaibi, Dey & Saha (2022) <doi:10.1155/2022/3135264>,
    Saha, Dey & Nadarajah (2022) <doi:10.1080/02664763.2021.1971632>,
    Saha, Tripathi & Dey (2024) <doi:10.1142/S021853932450013X>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.0.0)
Imports: stats, graphics, numDeriv, boot, MleCensoR
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-08-03 19:18:25 UTC; shikhar tyagi
Author: Shikhar Tyagi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1606-0844>),
  Sumit Kumar [aut],
  Arvind Pandey [aut],
  Bhupendra Singh [aut],
  Vrijesh Tripathi [aut]
Maintainer: Shikhar Tyagi <shikhar1093tyagi@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-08 13:00:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(autoplot,gpci_ci_censor)
S3method(autoplot,gpci_diagnostics_censor)
S3method(autoplot,gpcifit_censor)
S3method(coef,gpcifit_censor)
S3method(confint,gpcifit_censor)
S3method(plot,gpci_ci_censor)
S3method(plot,gpci_diagnostics_censor)
S3method(plot,gpcifit_censor)
S3method(print,gpci_ci_censor)
S3method(print,gpci_diagnostics_censor)
S3method(print,gpcifit_censor)
S3method(summary,gpci_ci_censor)
S3method(summary,gpci_diagnostics_censor)
S3method(summary,gpcifit_censor)
S3method(vcov,gpcifit_censor)
export(autoplot)
export(boot_ci_censor)
export(capability_censor)
export(compute_diagnostics_censor)
export(compute_theoretical_moments_censor)
export(define_distribution_censor)
export(dist_beta)
export(dist_gamma)
export(dist_logistic_exponential)
export(dist_lognormal)
export(dist_normal)
export(dist_weibull)
export(fit_distribution_censor)
export(gpci_boot_censor)
export(gpci_diagnostics_censor)
export(gpci_dist_censor)
export(gpci_fit_censor)
export(gpci_index)
importFrom(MleCensoR,mle_interval)
importFrom(boot,boot)
importFrom(boot,boot.ci)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(numDeriv,grad)
importFrom(numDeriv,hessian)
importFrom(stats,dbeta)
importFrom(stats,dgamma)
importFrom(stats,dlnorm)
importFrom(stats,dnorm)
importFrom(stats,dweibull)
importFrom(stats,integrate)
importFrom(stats,nlminb)
importFrom(stats,optim)
importFrom(stats,pbeta)
importFrom(stats,pgamma)
importFrom(stats,plnorm)
importFrom(stats,pnorm)
importFrom(stats,pweibull)
importFrom(stats,qbeta)
importFrom(stats,qgamma)
importFrom(stats,qlnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,qweibull)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(utils,globalVariables)

#' Define a Process Distribution for Interval-Censored Data
#'
#' Constructor to define a probability distribution for process capability analysis
#' under interval-censored data. The distribution can be defined by specifying any one
#' (or more) of its PDF/PMF, CDF, Survival Function (SF), or quantile function.
#' Missing functions are derived numerically.
#'
#' @param name Character string naming the distribution.
#' @param pdf Function representing the probability density function (PDF/PMF). Must be of the form \code{function(x, ...)}.
#' @param cdf Function representing the cumulative distribution function (CDF). Must be of the form \code{function(x, ...)}.
#' @param sf Function representing the survival function (SF = 1 - CDF). Must be of the form \code{function(x, ...)}.
#' @param quantile Function representing the quantile function. Must be of the form \code{function(p, ...)}.
#' @param params Named list of parameters for the distribution.
#' @param support Vector of length 2 defining the lower and upper bounds of the support of the distribution (default is \code{c(-Inf, Inf)}).
#' @return An object of class \code{gpci_dist_censor} representing the completed distribution.
#' @export
#' @examples
#' # Define custom Weibull distribution using only the survival function
#' custom_weib <- define_distribution_censor(
#'   name = "custom_weibull",
#'   sf = function(x, shape, scale) pweibull(x, shape, scale, lower.tail = FALSE),
#'   params = list(shape = 2, scale = 10),
#'   support = c(0, Inf)
#' )
define_distribution_censor <- function(name, pdf = NULL, cdf = NULL, sf = NULL, quantile = NULL, params = list(), support = c(-Inf, Inf)) {
  if (is.null(pdf) && is.null(cdf) && is.null(sf) && is.null(quantile)) {
    stop("At least one of 'pdf', 'cdf', 'sf', or 'quantile' function must be supplied.")
  }
  
  if (length(support) != 2 || support[1] >= support[2]) {
    stop("Invalid support limits. Must be a numeric vector of length 2 where support[1] < support[2].")
  }
  
  # Derive CDF from SF if CDF is missing
  if (is.null(cdf) && !is.null(sf)) {
    cdf <- function(x, ...) {
      1 - sf(x, ...)
    }
  }
  
  # Derive SF from CDF if SF is missing
  derived_sf <- sf
  if (is.null(derived_sf) && !is.null(cdf)) {
    derived_sf <- function(x, ...) {
      1 - cdf(x, ...)
    }
  }
  
  # Helper to find bracket for root-finding when support is infinite
  find_bracket <- function(cdf_fn, p, support_limits, ...) {
    lower <- support_limits[1]
    upper <- support_limits[2]
    if (is.na(lower) || is.infinite(lower)) lower <- -10
    if (is.na(upper) || is.infinite(upper)) upper <- 10
    
    iter <- 0
    while (iter < 30) {
      val_l <- tryCatch(do.call(cdf_fn, c(list(lower), list(...))), error = function(e) NA)
      if (!is.na(val_l[1]) && is.finite(val_l[1]) && val_l[1] < p) break
      lower <- lower * 2 - 10
      iter <- iter + 1
    }
    iter <- 0
    while (iter < 30) {
      val_u <- tryCatch(do.call(cdf_fn, c(list(upper), list(...))), error = function(e) NA)
      if (!is.na(val_u[1]) && is.finite(val_u[1]) && val_u[1] > p) break
      upper <- upper * 2 + 10
      iter <- iter + 1
    }
    if (is.na(lower) || !is.finite(lower)) lower <- -1e3
    if (is.na(upper) || !is.finite(upper)) upper <- 1e3
    if (lower >= upper) {
      lower <- -1e3
      upper <- 1e3
    }
    c(lower, upper)
  }
  
  derived_pdf <- pdf
  derived_cdf <- cdf
  derived_quantile <- quantile
  
  # Case A: PDF only is supplied
  if (!is.null(pdf) && is.null(cdf) && is.null(quantile)) {
    derived_cdf <- function(x, ...) {
      sapply(x, function(val) {
        if (val <= support[1]) return(0)
        if (val >= support[2]) return(1)
        safe_integrate_censor(function(v) pdf(v, ...), lower = support[1], upper = val)
      })
    }
    derived_quantile <- function(p, ...) {
      sapply(p, function(prob) {
        if (prob <= 0) return(support[1])
        if (prob >= 1) return(support[2])
        brac <- find_bracket(derived_cdf, prob, support, ...)
        safe_uniroot_censor(function(v) derived_cdf(v, ...) - prob, interval = brac)
      })
    }
  }
  
  # Case B: CDF only is supplied
  else if (is.null(pdf) && !is.null(cdf) && is.null(quantile)) {
    derived_pdf <- function(x, ...) {
      sapply(x, function(val) {
        if (val < support[1] || val > support[2]) return(0)
        max(0, safe_deriv_censor(function(v) cdf(v, ...), val))
      })
    }
    derived_quantile <- function(p, ...) {
      sapply(p, function(prob) {
        if (prob <= 0) return(support[1])
        if (prob >= 1) return(support[2])
        brac <- find_bracket(cdf, prob, support, ...)
        safe_uniroot_censor(function(v) cdf(v, ...) - prob, interval = brac)
      })
    }
  }
  
  # Case C: Quantile only is supplied
  else if (is.null(pdf) && is.null(cdf) && !is.null(quantile)) {
    derived_cdf <- function(x, ...) {
      sapply(x, function(val) {
        if (val <= support[1]) return(0)
        if (val >= support[2]) return(1)
        safe_uniroot_censor(function(p) quantile(p, ...) - val, interval = c(1e-9, 1 - 1e-9))
      })
    }
    derived_pdf <- function(x, ...) {
      sapply(x, function(val) {
        if (val < support[1] || val > support[2]) return(0)
        p <- derived_cdf(val, ...)
        if (p <= 1e-3 || p >= 1 - 1e-3) {
          return(max(0, safe_deriv_censor(function(v) derived_cdf(v, ...), val)))
        }
        dq <- safe_deriv_censor(function(prob) {
          prob_clamped <- max(1e-12, min(1 - 1e-12, prob))
          quantile(prob_clamped, ...)
        }, p)
        if (is.finite(dq) && dq > 1e-12) {
          return(1 / dq)
        } else {
          return(max(0, safe_deriv_censor(function(v) derived_cdf(v, ...), val)))
        }
      })
    }
  }
  
  # Case D: PDF and CDF supplied
  else if (!is.null(pdf) && !is.null(cdf) && is.null(quantile)) {
    derived_quantile <- function(p, ...) {
      sapply(p, function(prob) {
        if (prob <= 0) return(support[1])
        if (prob >= 1) return(support[2])
        brac <- find_bracket(cdf, prob, support, ...)
        safe_uniroot_censor(function(v) cdf(v, ...) - prob, interval = brac)
      })
    }
  }
  
  # Case E: CDF and Quantile supplied
  else if (is.null(pdf) && !is.null(cdf) && !is.null(quantile)) {
    derived_pdf <- function(x, ...) {
      sapply(x, function(val) {
        if (val < support[1] || val > support[2]) return(0)
        p <- cdf(val, ...)
        if (p <= 1e-3 || p >= 1 - 1e-3) {
          return(max(0, safe_deriv_censor(function(v) cdf(v, ...), val)))
        }
        dq <- safe_deriv_censor(function(prob) {
          prob_clamped <- max(1e-12, min(1 - 1e-12, prob))
          quantile(prob_clamped, ...)
        }, p)
        if (is.finite(dq) && dq > 1e-12) {
          return(1 / dq)
        } else {
          return(max(0, safe_deriv_censor(function(v) cdf(v, ...), val)))
        }
      })
    }
  }
  
  # Case F: PDF and Quantile supplied
  else if (!is.null(pdf) && is.null(cdf) && !is.null(quantile)) {
    derived_cdf <- function(x, ...) {
      sapply(x, function(val) {
        if (val <= support[1]) return(0)
        if (val >= support[2]) return(1)
        safe_uniroot_censor(function(p) quantile(p, ...) - val, interval = c(1e-9, 1 - 1e-9))
      })
    }
  }
  
  if (is.null(derived_sf)) {
    derived_sf <- function(x, ...) {
      1 - derived_cdf(x, ...)
    }
  }
  
  structure(
    list(
      name = name,
      pdf = derived_pdf,
      cdf = derived_cdf,
      sf = derived_sf,
      quantile = derived_quantile,
      params = params,
      support = support
    ),
    class = "gpci_dist_censor"
  )
}

#' @rdname define_distribution_censor
#' @export
gpci_dist_censor <- define_distribution_censor

# --- Built-in Distribution Constructors ---

#' Built-in Process Distributions for Interval-Censored Analysis
#'
#' Convenience constructors for standard process probability distributions.
#'
#' @param mean,sd Mean and standard deviation parameters for normal distribution.
#' @param meanlog,sdlog Mean and standard deviation of the distribution on the log scale.
#' @param shape,scale Shape and scale parameters.
#' @param shape1,shape2 Non-negative parameters of the Beta distribution.
#' @name builtin_dist
NULL

#' @rdname builtin_dist
#' @export
dist_normal <- function(mean = 0, sd = 1) {
  define_distribution_censor(
    name = "normal",
    pdf = function(x, mean, sd) stats::dnorm(x, mean, sd),
    cdf = function(x, mean, sd) stats::pnorm(x, mean, sd),
    sf = function(x, mean, sd) stats::pnorm(x, mean, sd, lower.tail = FALSE),
    quantile = function(p, mean, sd) stats::qnorm(p, mean, sd),
    params = list(mean = mean, sd = sd),
    support = c(-Inf, Inf)
  )
}

#' @rdname builtin_dist
#' @export
dist_lognormal <- function(meanlog = 0, sdlog = 1) {
  define_distribution_censor(
    name = "lognormal",
    pdf = function(x, meanlog, sdlog) stats::dlnorm(x, meanlog, sdlog),
    cdf = function(x, meanlog, sdlog) stats::plnorm(x, meanlog, sdlog),
    sf = function(x, meanlog, sdlog) stats::plnorm(x, meanlog, sdlog, lower.tail = FALSE),
    quantile = function(p, meanlog, sdlog) stats::qlnorm(p, meanlog, sdlog),
    params = list(meanlog = meanlog, sdlog = sdlog),
    support = c(0, Inf)
  )
}

#' @rdname builtin_dist
#' @export
dist_gamma <- function(shape = 1, scale = 1) {
  define_distribution_censor(
    name = "gamma",
    pdf = function(x, shape, scale) stats::dgamma(x, shape = shape, scale = scale),
    cdf = function(x, shape, scale) stats::pgamma(x, shape = shape, scale = scale),
    sf = function(x, shape, scale) stats::pgamma(x, shape = shape, scale = scale, lower.tail = FALSE),
    quantile = function(p, shape, scale) stats::qgamma(p, shape = shape, scale = scale),
    params = list(shape = shape, scale = scale),
    support = c(0, Inf)
  )
}

#' @rdname builtin_dist
#' @export
dist_weibull <- function(shape = 1, scale = 1) {
  define_distribution_censor(
    name = "weibull",
    pdf = function(x, shape, scale) stats::dweibull(x, shape = shape, scale = scale),
    cdf = function(x, shape, scale) stats::pweibull(x, shape = shape, scale = scale),
    sf = function(x, shape, scale) stats::pweibull(x, shape = shape, scale = scale, lower.tail = FALSE),
    quantile = function(p, shape, scale) stats::qweibull(p, shape = shape, scale = scale),
    params = list(shape = shape, scale = scale),
    support = c(0, Inf)
  )
}

#' @rdname builtin_dist
#' @export
dist_beta <- function(shape1 = 1, shape2 = 1) {
  define_distribution_censor(
    name = "beta",
    pdf = function(x, shape1, shape2) stats::dbeta(x, shape1, shape2),
    cdf = function(x, shape1, shape2) stats::pbeta(x, shape1, shape2),
    sf = function(x, shape1, shape2) stats::pbeta(x, shape1, shape2, lower.tail = FALSE),
    quantile = function(p, shape1, shape2) stats::qbeta(p, shape1, shape2),
    params = list(shape1 = shape1, shape2 = shape2),
    support = c(0, 1)
  )
}

#' @rdname builtin_dist
#' @export
dist_logistic_exponential <- function(shape = 1, scale = 1) {
  define_distribution_censor(
    name = "logistic_exponential",
    pdf = function(x, shape, scale) {
      ex <- exp(scale * x)
      num <- shape * scale * ex * (ex - 1)^(shape - 1)
      den <- (1 + (ex - 1)^shape)^2
      res <- num / den
      res[is.nan(res) | !is.finite(res)] <- 0
      res
    },
    cdf = function(x, shape, scale) {
      ex <- exp(scale * x)
      1 - 1 / (1 + (ex - 1)^shape)
    },
    sf = function(x, shape, scale) {
      ex <- exp(scale * x)
      1 / (1 + (ex - 1)^shape)
    },
    quantile = function(p, shape, scale) {
      (1 / scale) * log(1 + (p / (1 - p))^(1 / shape))
    },
    params = list(shape = shape, scale = scale),
    support = c(0, Inf)
  )
}

#' Compute Theoretical Moments of an Interval-Censored Distribution
#'
#' Computes the theoretical mean and variance of a \code{gpci_dist_censor} object using numerical integration.
#'
#' @param dist A \code{gpci_dist_censor} object.
#' @return A list with \code{mean} and \code{var}.
#' @export
compute_theoretical_moments_censor <- function(dist) {
  lower <- dist$support[1]
  upper <- dist$support[2]
  
  mean_fn <- function(x) {
    pdf_val <- tryCatch(do.call(dist$pdf, c(list(x), dist$params)), error = function(e) 0)
    if (is.na(pdf_val) || !is.finite(pdf_val)) pdf_val <- 0
    x * pdf_val
  }
  
  mean_val <- tryCatch({
    safe_integrate_censor(mean_fn, lower, upper)
  }, error = function(e) NA)
  
  if (is.na(mean_val) || !is.finite(mean_val)) {
    sim <- tryCatch({
      do.call(dist$quantile, c(list(runif(5000)), dist$params))
    }, error = function(e) {
      rep(mean(unlist(dist$params)), 5000)
    })
    sim[is.na(sim) | !is.finite(sim)] <- 0
    mean_val <- mean(sim)
    var_val <- stats::var(sim)
    return(list(mean = mean_val, var = max(var_val, 1e-10)))
  }
  
  var_fn <- function(x) {
    pdf_val <- tryCatch(do.call(dist$pdf, c(list(x), dist$params)), error = function(e) 0)
    if (is.na(pdf_val) || !is.finite(pdf_val)) pdf_val <- 0
    (x - mean_val)^2 * pdf_val
  }
  
  var_val <- tryCatch({
    safe_integrate_censor(var_fn, lower, upper)
  }, error = function(e) NA)
  
  if (is.na(var_val) || !is.finite(var_val) || var_val < 0) {
    sim <- tryCatch({
      do.call(dist$quantile, c(list(runif(5000)), dist$params))
    }, error = function(e) {
      rep(mean(unlist(dist$params)), 5000)
    })
    sim[is.na(sim) | !is.finite(sim)] <- 0
    var_val <- stats::var(sim)
  }
  
  var_val <- max(var_val, 1e-10)
  list(mean = mean_val, var = var_val)
}

test_that("define_distribution_censor creates valid distribution objects", {
  dist_norm <- dist_normal(mean = 10, sd = 2)
  expect_s3_class(dist_norm, "gpci_dist_censor")
  expect_equal(dist_norm$name, "normal")
  expect_equal(dist_norm$params$mean, 10)
  expect_equal(dist_norm$params$sd, 2)
  
  # Custom distribution using only SF
  custom_dist <- define_distribution_censor(
    name = "custom_exp",
    sf = function(x, rate) exp(-rate * x),
    params = list(rate = 0.5),
    support = c(0, Inf)
  )
  expect_s3_class(custom_dist, "gpci_dist_censor")
  expect_true(is.function(custom_dist$cdf))
  expect_true(is.function(custom_dist$pdf))
})

test_that("fit_distribution_censor estimates parameters under interval censoring", {
  set.seed(42)
  raw_x <- rnorm(30, mean = 10, sd = 1.5)
  left <- raw_x - 0.2
  right <- raw_x + 0.2
  
  dist_norm <- dist_normal(mean = 5, sd = 3)
  fitted <- fit_distribution_censor(left, right, dist_norm)
  
  expect_s3_class(fitted, "gpci_dist_censor")
  expect_true(is.finite(fitted$params$mean))
  expect_true(is.finite(fitted$params$sd))
  expect_true(fitted$params$sd > 0)
})

test_that("capability_censor computes GPCIs including Cpy without NAs", {
  set.seed(123)
  raw_x <- rnorm(25, mean = 100, sd = 5)
  left <- raw_x - 0.5
  right <- raw_x + 0.5
  
  dist_norm <- dist_normal(mean = 100, sd = 5)
  fit_cap <- capability_censor(
    data_left = left, data_right = right,
    distribution = dist_norm,
    USL = 115, LSL = 85, target = 100,
    indices = c("Cpy", "Cp", "Cpk", "Cpm", "Cpmk", "Spmk", "CpTk", "Cpc", "CNpmc", "CNpmkc"),
    mode = "moments"
  )
  
  expect_s3_class(fit_cap, "gpcifit_censor")
  estimates <- fit_cap$estimates
  expect_equal(length(estimates), 10)
  expect_false(any(is.na(estimates)))
  expect_false(any(is.nan(estimates)))
  expect_true(estimates["Cpy"] > 0)
  
  # Dispatcher test
  cpy_val <- gpci_index(fit_cap, "Cpy")
  expect_equal(unname(cpy_val), unname(estimates["Cpy"]))
})

test_that("boot_ci_censor computes CIs at 90%, 95%, 99% confidence levels", {
  set.seed(456)
  left <- c(9.5, 10.1, 8.9, 10.8, 9.7, 10.2, 9.4, 10.5)
  right <- c(10.3, 10.9, 9.5, 11.4, 10.3, 10.8, 10.0, 11.1)
  
  dist_norm <- dist_normal(mean = 10, sd = 1)
  fit_cap <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
  
  ci_res <- boot_ci_censor(fit_cap, B = 50, alpha = c(0.10, 0.05, 0.01), method = "percentile")
  expect_s3_class(ci_res, "gpci_ci_censor")
  expect_equal(nrow(ci_res$ci_table), 21) # 7 indices * 3 alpha levels
  expect_false(any(is.na(ci_res$ci_table$lower)))
  expect_false(any(is.na(ci_res$ci_table$upper)))
})

test_that("compute_diagnostics_censor computes SE, MSE, and coverage probabilities", {
  set.seed(789)
  left <- c(4.8, 5.1, 4.5, 5.3, 4.9)
  right <- c(5.2, 5.5, 4.9, 5.7, 5.3)
  
  dist_norm <- dist_normal(mean = 5, sd = 0.5)
  fit_cap <- capability_censor(left, right, dist_norm, USL = 7, LSL = 3)
  
  diag_res <- compute_diagnostics_censor(
    fit_cap,
    true_params = list(mean = 5, sd = 0.5),
    true_indices = c(Cpy = 1.0, Cp = 1.33),
    B = 30
  )
  
  expect_s3_class(diag_res, "gpci_diagnostics_censor")
  expect_true(length(diag_res$se_params) > 0)
  expect_true(length(diag_res$se_indices) > 0)
  expect_false(any(is.na(diag_res$se_params)))
  expect_false(any(is.na(diag_res$se_indices)))
})

test_that("S3 methods function correctly", {
  set.seed(101)
  left <- c(9.8, 10.1, 9.5)
  right <- c(10.2, 10.5, 9.9)
  dist_norm <- dist_normal()
  fit <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
  
  expect_output(print(fit))
  expect_output(summary(fit))
  expect_type(coef(fit, "indices"), "double")
  expect_type(coef(fit, "parameters"), "double")
})

#' Compute Standard Errors, MSE, and Coverage Probabilities for Interval-Censored Data
#'
#' Computes standard errors, mean squared errors, and coverage probabilities for
#' distribution parameters and process capability indices using bootstrap results.
#'
#' @param fit A \code{gpcifit_censor} object returned by \code{\link{capability_censor}}.
#' @param true_params Named list of true parameter values (if known, for simulation studies).
#' @param true_indices Named vector of true index values (if known, for simulation studies).
#' @param B Number of bootstrap replicates for SE computation (default is 1000).
#' @param alpha Vector of significance levels for coverage probability (default is \code{c(0.10, 0.05, 0.01)}).
#' @return A list of class \code{"gpci_diagnostics_censor"} containing standard errors, MSE, and coverage probabilities.
#' @export
#' @examples
#' \dontrun{
#' dist_norm <- dist_normal()
#' left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
#' right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
#' fit <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
#' diagnostics <- compute_diagnostics_censor(fit, B = 500)
#' print(diagnostics)
#' }
compute_diagnostics_censor <- function(fit, true_params = NULL, true_indices = NULL, 
                                        B = 1000, alpha = c(0.10, 0.05, 0.01)) {
  if (!inherits(fit, "gpcifit_censor")) {
    stop("fit must be of class 'gpcifit_censor'")
  }
  
  boot_res <- boot_ci_censor(fit, B = B, alpha = alpha, method = "percentile")
  
  indices <- fit$indices
  boot_t <- boot_res$boot_out$t
  
  se_indices <- sapply(seq_along(indices), function(i) {
    reps <- boot_t[, i]
    reps <- reps[is.finite(reps)]
    if (length(reps) > 0) {
      val <- stats::sd(reps, na.rm = TRUE)
      if (is.na(val) || !is.finite(val)) val <- 0
      val
    } else {
      0
    }
  })
  names(se_indices) <- indices
  
  mse_indices <- NULL
  if (!is.null(true_indices)) {
    mse_indices <- sapply(indices, function(idx) {
      if (idx %in% names(true_indices)) {
        est <- fit$estimates[idx]
        true_val <- true_indices[idx]
        val <- (est - true_val)^2
        if (is.na(val) || !is.finite(val)) val <- 0
        val
      } else {
        0
      }
    })
  }
  
  coverage_indices <- list()
  for (a in alpha) {
    coverage <- sapply(seq_along(indices), function(i) {
      reps <- boot_t[, i]
      reps <- reps[is.finite(reps)]
      if (length(reps) > 0 && !is.null(true_indices) && indices[i] %in% names(true_indices)) {
        ci_lower <- stats::quantile(reps, probs = a / 2, na.rm = TRUE)
        ci_upper <- stats::quantile(reps, probs = 1 - a / 2, na.rm = TRUE)
        true_val <- true_indices[indices[i]]
        as.numeric(true_val >= ci_lower && true_val <= ci_upper)
      } else {
        0
      }
    })
    names(coverage) <- indices
    coverage_indices[[paste0("alpha_", a)]] <- coverage
  }
  
  param_names <- names(fit$distribution$params)
  se_params <- sapply(param_names, function(pname) {
    n <- length(fit$data_left)
    jack_est <- numeric(n)
    for (i in 1:n) {
      left_j <- fit$data_left[-i]
      right_j <- fit$data_right[-i]
      fit_j <- tryCatch({
        fit_distribution_censor(left_j, right_j, fit$distribution)
      }, error = function(e) NULL)
      
      if (!is.null(fit_j)) {
        jack_est[i] <- fit_j$params[[pname]]
      } else {
        jack_est[i] <- fit$distribution$params[[pname]]
      }
    }
    jack_est <- jack_est[is.finite(jack_est)]
    if (length(jack_est) > 0) {
      val <- sqrt((n - 1) / n * stats::var(jack_est))
      if (is.na(val) || !is.finite(val)) val <- 0
      val
    } else {
      0
    }
  })
  
  mse_params <- NULL
  if (!is.null(true_params)) {
    mse_params <- sapply(param_names, function(pname) {
      if (pname %in% names(true_params)) {
        est <- fit$distribution$params[[pname]]
        true_val <- true_params[[pname]]
        val <- (est - true_val)^2
        if (is.na(val) || !is.finite(val)) val <- 0
        val
      } else {
        0
      }
    })
  }
  
  coverage_params <- list()
  if (!is.null(true_params)) {
    for (a in alpha) {
      coverage <- sapply(param_names, function(pname) {
        if (pname %in% names(true_params)) {
          est <- fit$distribution$params[[pname]]
          se <- se_params[pname]
          if (is.finite(se) && se > 0) {
            ci_lower <- est - stats::qnorm(1 - a / 2) * se
            ci_upper <- est + stats::qnorm(1 - a / 2) * se
            true_val <- true_params[[pname]]
            as.numeric(true_val >= ci_lower && true_val <= ci_upper)
          } else {
            0
          }
        } else {
          0
        }
      })
      names(coverage) <- param_names
      coverage_params[[paste0("alpha_", a)]] <- coverage
    }
  }
  
  structure(
    list(
      se_indices = se_indices,
      mse_indices = mse_indices,
      coverage_indices = coverage_indices,
      se_params = se_params,
      mse_params = mse_params,
      coverage_params = coverage_params,
      alpha_levels = alpha,
      bootstrap_replicates = B
    ),
    class = "gpci_diagnostics_censor"
  )
}

#' @rdname compute_diagnostics_censor
#' @export
gpci_diagnostics_censor <- compute_diagnostics_censor

#' Print Method for gpci_diagnostics_censor
#'
#' @param x An object of class \code{gpci_diagnostics_censor}.
#' @param ... Additional print arguments.
#' @export
print.gpci_diagnostics_censor <- function(x, ...) {
  cat("--- Diagnostics for Interval-Censored Data ---\n")
  cat("Bootstrap Replicates:", x$bootstrap_replicates, "\n\n")
  
  cat("Standard Errors for Parameters:\n")
  print(round(x$se_params, 6))
  
  if (!is.null(x$mse_params)) {
    cat("\nMean Squared Errors for Parameters:\n")
    print(round(x$mse_params, 6))
  }
  
  cat("\nStandard Errors for Capability Indices:\n")
  print(round(x$se_indices, 6))
  
  if (!is.null(x$mse_indices)) {
    cat("\nMean Squared Errors for Capability Indices:\n")
    print(round(x$mse_indices, 6))
  }
  
  if (length(x$coverage_indices) > 0) {
    cat("\nCoverage Probabilities for Capability Indices:\n")
    for (a_name in names(x$coverage_indices)) {
      cat("\nAlpha level:", a_name, "\n")
      print(round(x$coverage_indices[[a_name]], 4))
    }
  }
  
  if (length(x$coverage_params) > 0) {
    cat("\nCoverage Probabilities for Parameters:\n")
    for (a_name in names(x$coverage_params)) {
      cat("\nAlpha level:", a_name, "\n")
      print(round(x$coverage_params[[a_name]], 4))
    }
  }
  
  invisible(x)
}

#' Summary Method for gpci_diagnostics_censor
#'
#' @param object An object of class \code{gpci_diagnostics_censor}.
#' @param ... Additional arguments.
#' @export
summary.gpci_diagnostics_censor <- function(object, ...) {
  print(object, ...)
  invisible(object)
}

if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    "Cpc", "alpha_factor", "avg_width", "cdf", "density", "empirical",
    "empirical_coverage", "estimate", "hpd_95_lower", "hpd_95_upper", "index",
    "iter", "lower", "nominal_coverage", "p0", "pdf", "posterior_mean",
    "ppoints", "replicate", "runif", "stack", "theoretical", "upper", "value", "x",
    "data_left", "data_right", "left", "right"
  ))
}

#' Safe Numerical Integration for Interval-Censored Analysis
#'
#' A robust wrapper around \code{stats::integrate} that handles vectorization,
#' infinite bounds, and numerical exceptions gracefully.
#'
#' @param f A function to integrate.
#' @param lower Lower limit of integration.
#' @param upper Upper limit of integration.
#' @param ... Additional arguments passed to \code{f}.
#' @return A numeric value representing the integral.
#' @keywords internal
safe_integrate_censor <- function(f, lower, upper, ...) {
  vf <- Vectorize(function(x) {
    val <- tryCatch(f(x, ...), error = function(e) 0)
    if (is.na(val) || !is.finite(val)) 0 else val
  })
  
  res <- tryCatch({
    stats::integrate(vf, lower = lower, upper = upper, rel.tol = 1e-8, stop.on.error = FALSE)
  }, error = function(e) {
    NULL
  })
  
  if (is.null(res) || inherits(res, "error") || (res$message != "OK" && res$message != "subdivisions limit exceeded")) {
    if (is.finite(lower) && is.finite(upper)) {
      grid <- seq(lower, upper, length.out = 1000)
      y <- vf(grid)
      w <- (upper - lower) / 999
      val <- sum(y[-1] + y[-length(y)]) * w / 2
      return(if (is.na(val) || !is.finite(val)) 0 else val)
    } else {
      return(0)
    }
  }
  val <- res$value
  if (is.na(val) || !is.finite(val)) 0 else val
}

#' Safe Root Finding for Quantile Inversion
#'
#' A robust wrapper around \code{stats::uniroot} that finds roots for CDF or quantile inversion.
#'
#' @param f A function whose root is to be found.
#' @param interval A vector containing the end-points of the interval to be searched.
#' @param ... Additional arguments passed to \code{f}.
#' @return A numeric value of the root.
#' @keywords internal
safe_uniroot_censor <- function(f, interval, ...) {
  if (any(is.na(interval)) || any(!is.finite(interval)) || interval[1] >= interval[2]) {
    interval <- c(-1e3, 1e3)
  }
  res <- tryCatch({
    stats::uniroot(f, interval = interval, tol = 1e-8, extendInt = "yes")$root
  }, error = function(e) {
    x_grid <- seq(interval[1], interval[2], length.out = 100)
    y_grid <- sapply(x_grid, function(val) {
      v <- tryCatch(f(val, ...), error = function(err) NA)
      if (is.na(v) || !is.finite(v)) 1e10 else v
    })
    signs <- sign(y_grid)
    zero_crossings <- which(signs[-1] != signs[-length(signs)])
    if (length(zero_crossings) > 0) {
      idx <- zero_crossings[1]
      tryCatch({
        stats::uniroot(f, interval = c(x_grid[idx], x_grid[idx+1]), tol = 1e-8)$root
      }, error = function(err) {
        x_grid[which.min(abs(y_grid))]
      })
    } else {
      x_grid[which.min(abs(y_grid))]
    }
  })
  if (is.na(res) || !is.finite(res)) mean(interval) else res
}

#' Safe Numerical Differentiation
#'
#' A wrapper around \code{numDeriv::grad} with fallback to finite differences.
#'
#' @param f A function to differentiate.
#' @param x The point(s) at which to differentiate.
#' @param ... Additional arguments passed to \code{f}.
#' @return The numerical derivative at \code{x}.
#' @keywords internal
safe_deriv_censor <- function(f, x, ...) {
  res <- tryCatch({
    numDeriv::grad(f, x, ...)
  }, error = function(e) {
    h <- 1e-5
    (f(x + h, ...) - f(x - h, ...)) / (2 * h)
  })
  if (is.na(res) || !is.finite(res)) 0 else res
}

#' Helper to Round Numeric Columns of a Data Frame
#'
#' @param x A data frame.
#' @param digits Integer specifying digits.
#' @return Data frame with rounded numeric columns.
#' @keywords internal
round_df_censor <- function(x, digits) {
  numeric_cols <- sapply(x, is.numeric)
  x[numeric_cols] <- lapply(x[numeric_cols], round, digits = digits)
  x
}

#' Compute Process Capability Indices for Interval-Censored Data
#'
#' Computes classical and generalized Process Capability Indices (PCIs) for interval-censored data.
#'
#' @param data_left Numeric vector of left interval bounds for interval-censored data.
#' @param data_right Numeric vector of right interval bounds for interval-censored data.
#' @param distribution A \code{gpci_dist_censor} distribution object.
#' @param USL Numeric value of the Upper Specification Limit.
#' @param LSL Numeric value of the Lower Specification Limit.
#' @param target Numeric value of the process target (defaults to the midpoint of USL and LSL).
#' @param indices Character vector of indices to compute. Choices include Vännman's classical family
#'   (\code{"Cp"}, \code{"Cpk"}, \code{"Cpl"}, \code{"Cpu"}, \code{"Cpm"}, \code{"Cpmk"}, \code{"Cp_uv"})
#'   and their quantile analogs (\code{"Cp_q"}, \code{"Cpk_q"}, \code{"Cpl_q"}, \code{"Cpu_q"}, \code{"CNpk"}, \code{"Cpm_q"}, \code{"Cpmk_q"}, \code{"CNp_uv"}),
#'   as well as generalized indices from literature (\code{"Cpy"}, \code{"CpTk"}, \code{"Spmk"}, \code{"Cpc"}, \code{"CNpmc"}, \code{"CNpmkc"}).
#' @param u Parameter \code{u} for the generalized \code{Cp(u, v)} family (default is 1).
#' @param v Parameter \code{v} for the generalized \code{Cp(u, v)} family (default is 1).
#' @param mode Mode of computation: \code{"moments"} (uses mean and variance) or \code{"quantile"} (uses robust quantiles).
#' @param fit Logical. If \code{TRUE} (default) and data is supplied, parameter values of the distribution are estimated from interval-censored data via MLE.
#' @param C0,C1,C2 Coefficients for the tolerance cost function in \code{CNpmc} and \code{CNpmkc} (defaults: \code{C0 = 1, C1 = 0, C2 = 1}).
#' @param tolerance_t Process tolerance \code{t} for the tolerance cost function (defaults to \code{USL - LSL}).
#' @param P0 Desirable yield for the \code{Cpc} and \code{Cpy} indices (default is 0.9973002).
#' @param LDL,UDL Lower and Upper Desired Limits for the \code{CpTk} index (default to \code{LSL} and \code{USL} respectively).
#' @return An object of class \code{"gpcifit_censor"} containing raw data, specification limits, fitted parameters, and computed indices.
#' @export
#' @examples
#' \dontrun{
#' dist_norm <- dist_normal()
#' left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
#' right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
#' capability_censor(
#'   data_left = left, data_right = right,
#'   distribution = dist_norm,
#'   USL = 13, LSL = 7, target = 10,
#'   indices = c("Cpy", "Cp", "Cpk", "Cpm", "Cpmk"),
#'   mode = "moments"
#' )
#' }
capability_censor <- function(data_left = NULL, data_right = NULL, distribution, USL, LSL, target = (USL + LSL)/2,
                               indices = c("Cpy", "Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk"),
                               u = 1, v = 1, mode = c("moments", "quantile"),
                               fit = TRUE,
                               C0 = 1, C1 = 0, C2 = 1, tolerance_t = USL - LSL,
                               P0 = 0.9973002, LDL = LSL, UDL = USL) {
  mode <- match.arg(mode)
  
  if (!inherits(distribution, "gpci_dist_censor")) {
    stop("distribution must be of class 'gpci_dist_censor'")
  }
  
  if (missing(USL) || missing(LSL)) {
    stop("Both USL and LSL must be specified.")
  }
  
  if (LSL >= USL) {
    stop("LSL must be less than USL.")
  }
  
  if (LSL < distribution$support[1] || USL > distribution$support[2]) {
    warning("Specification limits exceed the support of the distribution.")
  }
  
  dist_fitted <- distribution
  if (fit && !is.null(data_left) && !is.null(data_right)) {
    dist_fitted <- fit_distribution_censor(data_left, data_right, distribution)
  }
  
  moments <- compute_theoretical_moments_censor(dist_fitted)
  mu <- moments$mean
  sigma <- sqrt(moments$var)
  
  if (is.na(mu) || !is.finite(mu)) mu <- mean(unlist(dist_fitted$params))
  if (is.na(sigma) || !is.finite(sigma) || sigma <= 0) sigma <- 1
  
  q_00135 <- tryCatch(do.call(dist_fitted$quantile, c(list(0.00135), dist_fitted$params)), error = function(e) mu - 3*sigma)
  q_05 <- tryCatch(do.call(dist_fitted$quantile, c(list(0.5), dist_fitted$params)), error = function(e) mu)
  q_99865 <- tryCatch(do.call(dist_fitted$quantile, c(list(0.99865), dist_fitted$params)), error = function(e) mu + 3*sigma)
  
  q_00135[is.na(q_00135) | !is.finite(q_00135)] <- mu - 3*sigma
  q_05[is.na(q_05) | !is.finite(q_05)] <- mu
  q_99865[is.na(q_99865) | !is.finite(q_99865)] <- mu + 3*sigma
  
  d <- (USL - LSL) / 2
  M_mid <- (USL + LSL) / 2
  
  CM_t <- C0 + C1 * exp(-C2 * tolerance_t)
  if (is.na(CM_t) || !is.finite(CM_t)) CM_t <- 1
  
  F_USL <- tryCatch(do.call(dist_fitted$cdf, c(list(USL), dist_fitted$params)), error = function(e) 0.99)
  F_LSL <- tryCatch(do.call(dist_fitted$cdf, c(list(LSL), dist_fitted$params)), error = function(e) 0.01)
  
  F_USL[is.na(F_USL) | !is.finite(F_USL)] <- 0.99
  F_LSL[is.na(F_LSL) | !is.finite(F_LSL)] <- 0.01
  F_USL <- max(0, min(1, F_USL))
  F_LSL <- max(0, min(1, F_LSL))
  
  yield <- F_USL - F_LSL
  yield <- max(1e-12, min(1 - 1e-12, yield))
  p_hat <- 1 - yield
  
  estimates <- numeric(0)
  
  mu_mode <- if (mode == "moments") mu else q_05
  sigma_mode <- if (mode == "moments") sigma else (q_99865 - q_00135) / 6
  if (is.na(sigma_mode) || !is.finite(sigma_mode) || sigma_mode <= 0) sigma_mode <- sigma
  
  for (idx in indices) {
    val <- tryCatch(switch(idx,
      "Cpy" = yield / P0,
      "Cp" = d / (3 * sigma_mode),
      "Cpk" = min(USL - mu_mode, mu_mode - LSL) / (3 * sigma_mode),
      "Cpu" = (USL - mu_mode) / (3 * sigma_mode),
      "Cpl" = (mu_mode - LSL) / (3 * sigma_mode),
      "Cpm" = d / (3 * sqrt(sigma_mode^2 + (mu_mode - target)^2)),
      "Cpmk" = min(USL - mu_mode, mu_mode - LSL) / (3 * sqrt(sigma_mode^2 + (mu_mode - target)^2)),
      "Cp_uv" = (d - u * abs(mu_mode - M_mid)) / (3 * sqrt(sigma_mode^2 + v * (mu_mode - target)^2)),
      
      # Quantile analogs
      "Cp_q" = (USL - LSL) / max(q_99865 - q_00135, 1e-10),
      "Cpl_q" = (q_05 - LSL) / max(q_05 - q_00135, 1e-10),
      "Cpu_q" = (USL - q_05) / max(q_99865 - q_05, 1e-10),
      "Cpk_q" = min((USL - q_05) / max(q_99865 - q_05, 1e-10), (q_05 - LSL) / max(q_05 - q_00135, 1e-10)),
      "CNpk" = (d - abs(q_05 - M_mid)) / max((q_99865 - q_00135) / 2, 1e-10),
      "CNp_uv" = (d - u * abs(q_05 - M_mid)) / (3 * sqrt(max(((q_99865 - q_00135) / 6)^2 + v * (q_05 - target)^2, 1e-10))),
      "Cpm_q" = (USL - LSL) / (6 * sqrt(max(((q_99865 - q_00135) / 6)^2 + (q_05 - target)^2, 1e-10))),
      "Cpmk_q" = (d - abs(q_05 - M_mid)) / (3 * sqrt(max(((q_99865 - q_00135) / 6)^2 + (q_05 - target)^2, 1e-10))),
      
      # Generalized Indices from literature
      "CpTk" = {
        F_T <- tryCatch(do.call(dist_fitted$cdf, c(list(target), dist_fitted$params)), error = function(e) 0.5)
        alpha1 <- tryCatch(do.call(dist_fitted$cdf, c(list(LDL), dist_fitted$params)), error = function(e) 0.01)
        alpha2 <- 1 - tryCatch(do.call(dist_fitted$cdf, c(list(UDL), dist_fitted$params)), error = function(e) 0.99)
        
        den2 <- 0.5 - alpha2
        den1 <- 0.5 - alpha1
        term2 <- if (den2 > 1e-10) (F_USL - F_T) / den2 else 1e5
        term1 <- if (den1 > 1e-10) (F_T - F_LSL) / den1 else 1e5
        min(term2, term1)
      },
      "Spmk" = {
        val_inside <- (1 + F_USL - F_LSL) / 2
        val_inside <- max(1e-9, min(1 - 1e-9, val_inside))
        qnorm_val <- tryCatch(stats::qnorm(val_inside), error = function(e) 0)
        qnorm_val / (3 * sqrt(max(1 + ((mu - target) / sigma)^2, 1e-10)))
      },
      "Cpc" = {
        if (yield >= 1) 1e5 else (1 - P0) / max(1e-9, 1 - yield)
      },
      "CNpmc" = {
        (USL - LSL) / (6 * sqrt(max(((q_99865 - q_00135) / 6)^2 + (q_05 - target)^2 + CM_t, 1e-10)))
      },
      "CNpmkc" = {
        psi <- sqrt(max(((q_99865 - q_00135) / 6)^2 + (q_05 - target)^2, 1e-10))
        cn_u <- (USL - M_mid) / (3 * sqrt(max(psi^2 + CM_t, 1e-10)))
        cn_l <- (M_mid - LSL) / (3 * sqrt(max(psi^2 + CM_t, 1e-10)))
        min(cn_u, cn_l)
      },
      0
    ), error = function(e) 0)
    
    if (is.na(val) || !is.finite(val)) val <- 0
    estimates[idx] <- val
  }
  
  structure(
    list(
      data_left = data_left,
      data_right = data_right,
      distribution = dist_fitted,
      USL = USL,
      LSL = LSL,
      target = target,
      indices = indices,
      estimates = estimates,
      p_hat = p_hat,
      mode = mode,
      options = list(
        u = u, v = v, C0 = C0, C1 = C1, C2 = C2,
        tolerance_t = tolerance_t, P0 = P0, LDL = LDL, UDL = UDL
      )
    ),
    class = "gpcifit_censor"
  )
}

#' Dispatcher for Single Index Computation under Interval Censoring
#'
#' @param fit A \code{gpcifit_censor} object.
#' @param index Name of index to compute.
#' @return Numeric value of the specified index.
#' @export
gpci_index <- function(fit, index) {
  if (!inherits(fit, "gpcifit_censor")) stop("fit must be of class 'gpcifit_censor'")
  if (index %in% names(fit$estimates)) {
    return(fit$estimates[[index]])
  }
  cap <- capability_censor(
    data_left = fit$data_left, data_right = fit$data_right,
    distribution = fit$distribution,
    USL = fit$USL, LSL = fit$LSL, target = fit$target,
    indices = index, u = fit$options$u, v = fit$options$v,
    mode = fit$mode, fit = FALSE,
    C0 = fit$options$C0, C1 = fit$options$C1, C2 = fit$options$C2,
    tolerance_t = fit$options$tolerance_t, P0 = fit$options$P0,
    LDL = fit$options$LDL, UDL = fit$options$UDL
  )
  cap$estimates[[index]]
}

#' Print Method for gpcifit_censor
#'
#' @param x An object of class \code{gpcifit_censor}.
#' @param ... Additional print arguments.
#' @export
print.gpcifit_censor <- function(x, ...) {
  cat("--- Process Capability Analysis for Interval-Censored Data ---\n")
  cat("Distribution: ", x$distribution$name, "\n")
  cat("Parameters:   ", paste(names(x$distribution$params), "=", round(unlist(x$distribution$params), 4), collapse = ", "), "\n")
  cat("Spec Limits:  LSL =", x$LSL, ", USL =", x$USL, ", Target =", x$target, "\n")
  cat("Mode:         ", x$mode, "\n")
  cat("Expected Nonconforming (p_hat): ", round(x$p_hat * 100, 4), "%\n\n")
  cat("Point Estimates of Capability Indices:\n")
  print(round(x$estimates, 4))
  invisible(x)
}

#' Summary Method for gpcifit_censor
#'
#' @param object An object of class \code{gpcifit_censor}.
#' @param ... Additional arguments.
#' @export
summary.gpcifit_censor <- function(object, ...) {
  print(object, ...)
  invisible(object)
}

#' Coef Method for gpcifit_censor
#'
#' Extract point estimates of capability indices or fitted distribution parameters.
#'
#' @param object An object of class \code{gpcifit_censor}.
#' @param what Character string: \code{"indices"} (default) or \code{"parameters"}.
#' @param ... Additional arguments.
#' @export
coef.gpcifit_censor <- function(object, what = c("indices", "parameters"), ...) {
  what <- match.arg(what)
  if (what == "indices") {
    object$estimates
  } else {
    unlist(object$distribution$params)
  }
}

#' Vcov Method for gpcifit_censor
#'
#' Extract asymptotic variance-covariance matrix of fitted parameters.
#'
#' @param object An object of class \code{gpcifit_censor}.
#' @param ... Additional arguments.
#' @export
vcov.gpcifit_censor <- function(object, ...) {
  v <- attr(object$distribution, "vcov")
  if (is.null(v)) {
    warning("Variance-covariance matrix not available for this fit.")
    return(NULL)
  }
  v
}

#' Confint Method for gpcifit_censor
#'
#' Calculate confidence intervals for capability indices via bootstrap at specified significance levels (90%, 95%, 99%).
#'
#' @param object An object of class \code{gpcifit_censor}.
#' @param parm Optional vector of index names.
#' @param level Confidence level (default is 0.95; can be 0.90, 0.95, 0.99 or a vector).
#' @param B Number of bootstrap replicates (default 1000).
#' @param method Bootstrap CI method (\code{"percentile"}, \code{"normal"}, \code{"basic"}, \code{"BCa"}, \code{"BCp"}, \code{"studentized"}).
#' @param ... Additional arguments.
#' @export
confint.gpcifit_censor <- function(object, parm = NULL, level = 0.95, B = 1000, method = "percentile", ...) {
  alpha_val <- 1 - level
  ci_res <- boot_ci_censor(fit = object, B = B, alpha = alpha_val, method = method, ...)
  tbl <- ci_res$ci_table
  if (!is.null(parm)) {
    tbl <- tbl[tbl$index %in% parm, ]
  }
  res_mat <- as.matrix(tbl[, c("lower", "upper")])
  rownames(res_mat) <- paste(tbl$index, paste0("(alpha=", tbl$alpha, ")"))
  colnames(res_mat) <- c("Lower CI", "Upper CI")
  res_mat
}

#' Compute Bootstrap Confidence Intervals for Process Capability Indices (Interval-Censored Data)
#'
#' Runs parametric or non-parametric bootstrapping to compute confidence intervals
#' for generalized process capability indices at multiple significance levels (90%, 95%, 99%)
#' under interval-censored data.
#'
#' @param fit A \code{gpcifit_censor} object returned by \code{\link{capability_censor}}.
#' @param B Number of bootstrap replicates (default is 2000).
#' @param alpha Vector of significance levels (default is \code{c(0.10, 0.05, 0.01)} corresponding to 90%, 95%, and 99% confidence levels).
#' @param method Confidence interval method. Choices are \code{"percentile"}, \code{"normal"},
#'   \code{"basic"}, \code{"BCa"}, \code{"BCp"} (Bias-Corrected Percentile), or \code{"studentized"}.
#' @param type Bootstrap type: \code{"parametric"} or \code{"nonparametric"}.
#' @param parallel Logical. If \code{TRUE}, bootstrap replicates are run in parallel.
#' @param ncpus Integer. Number of CPUs to use if \code{parallel = TRUE}.
#' @return An object of class \code{gpci_ci_censor} containing a tidy data frame of confidence intervals,
#'   the raw bootstrap output, and the original fit object.
#' @export
#' @examples
#' \dontrun{
#' dist_norm <- dist_normal()
#' left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
#' right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
#' fit <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
#' ci <- boot_ci_censor(fit, B = 500, alpha = c(0.10, 0.05, 0.01), method = "percentile")
#' print(ci)
#' }
boot_ci_censor <- function(fit, B = 2000, alpha = c(0.10, 0.05, 0.01),
                           method = c("percentile", "normal", "basic", "BCa", "BCp", "studentized"),
                           type = c("parametric", "nonparametric"),
                           parallel = FALSE, ncpus = 1) {
  method <- match.arg(method)
  type <- match.arg(type)
  
  if (!inherits(fit, "gpcifit_censor")) {
    stop("fit must be of class 'gpcifit_censor'")
  }
  
  if (is.null(fit$data_left) || is.null(fit$data_right)) {
    stop("Bootstrap confidence intervals require interval-censored data to resample.")
  }
  
  n <- length(fit$data_left)
  indices <- fit$indices
  
  p_type <- "no"
  if (parallel) {
    p_type <- if (.Platform$OS.type == "windows") "snow" else "multicore"
  }
  
  # Function to generate nonparametric bootstrap samples for interval-censored pairs
  generate_bootstrap_sample <- function(data_left, data_right) {
    n_samp <- length(data_left)
    sample_idx <- sample(1:n_samp, n_samp, replace = TRUE)
    list(left = data_left[sample_idx], right = data_right[sample_idx])
  }
  
  # Function to generate parametric bootstrap samples
  generate_parametric_sample <- function(n_samp, dist) {
    u <- stats::runif(n_samp)
    samples <- tryCatch(do.call(dist$quantile, c(list(u), dist$params)), error = function(e) {
      mu <- mean(unlist(dist$params))
      stats::qnorm(u, mu, 1)
    })
    samples[is.na(samples) | !is.finite(samples)] <- mean(unlist(dist$params))
    
    vcov_mat <- attr(dist, "vcov")
    interval_width <- 0.1
    if (!is.null(vcov_mat) && is.matrix(vcov_mat) && nrow(vcov_mat) > 0) {
      diag_max <- suppressWarnings(max(diag(vcov_mat), na.rm = TRUE))
      if (is.finite(diag_max) && diag_max > 0) {
        interval_width <- 0.1 * sqrt(diag_max)
      }
    }
    interval_width <- max(interval_width, 0.01)
    list(left = samples - interval_width / 2, right = samples + interval_width / 2)
  }
  
  # Jackknife standard error vector for studentized bootstrap
  jackknife_se_vector <- function(left, right) {
    n_y <- length(left)
    jack_est <- matrix(0, nrow = n_y, ncol = length(indices))
    for (i in 1:n_y) {
      left_j <- left[-i]
      right_j <- right[-i]
      fit_j <- tryCatch({
        capability_censor(
          data_left = left_j, data_right = right_j,
          distribution = fit$distribution,
          USL = fit$USL, LSL = fit$LSL, target = fit$target,
          indices = indices, u = fit$options$u, v = fit$options$v,
          mode = fit$mode, fit = TRUE,
          C0 = fit$options$C0, C1 = fit$options$C1, C2 = fit$options$C2,
          tolerance_t = fit$options$tolerance_t, P0 = fit$options$P0,
          LDL = fit$options$LDL, UDL = fit$options$UDL
        )
      }, error = function(e) NULL)
      
      if (!is.null(fit_j)) {
        jack_est[i, ] <- fit_j$estimates
      }
    }
    col_vars <- apply(jack_est, 2, function(col) {
      m_col <- mean(col, na.rm = TRUE)
      (n_y - 1) / n_y * sum((col - m_col)^2, na.rm = TRUE)
    })
    sqrt(pmax(1e-12, col_vars))
  }
  
  # Statistic function for boot::boot
  stat_fn <- function(d, i) {
    if (type == "nonparametric") {
      boot_data <- generate_bootstrap_sample(fit$data_left, fit$data_right)
      left_boot <- boot_data$left
      right_boot <- boot_data$right
    } else {
      boot_data <- generate_parametric_sample(n, fit$distribution)
      left_boot <- boot_data$left
      right_boot <- boot_data$right
    }
    
    fit_star <- tryCatch({
      capability_censor(
        data_left = left_boot, data_right = right_boot,
        distribution = fit$distribution,
        USL = fit$USL, LSL = fit$LSL, target = fit$target,
        indices = indices, u = fit$options$u, v = fit$options$v,
        mode = fit$mode, fit = TRUE,
        C0 = fit$options$C0, C1 = fit$options$C1, C2 = fit$options$C2,
        tolerance_t = fit$options$tolerance_t, P0 = fit$options$P0,
        LDL = fit$options$LDL, UDL = fit$options$UDL
      )
    }, error = function(e) NULL)
    
    if (is.null(fit_star)) {
      return(rep(0, length(indices)))
    }
    
    estimates <- fit_star$estimates
    estimates[is.na(estimates) | !is.finite(estimates)] <- 0
    
    if (method == "studentized") {
      se <- jackknife_se_vector(left_boot, right_boot)
      se[is.na(se) | !is.finite(se)] <- 0
      return(c(estimates, se^2))
    }
    
    return(estimates)
  }
  
  # Run bootstrap
  boot_out <- boot::boot(
    data = data.frame(left = fit$data_left, right = fit$data_right),
    statistic = stat_fn,
    R = B,
    parallel = p_type,
    ncpus = ncpus
  )
  
  ci_list <- list()
  for (idx_name in indices) {
    idx_num <- which(indices == idx_name)
    point_est <- fit$estimates[idx_name]
    reps <- boot_out$t[, idx_num]
    reps_clean <- reps[is.finite(reps)]
    if (length(reps_clean) < 5) reps_clean <- rep(point_est, 10)
    
    for (a in alpha) {
      conf <- 1 - a
      
      bounds <- tryCatch({
        if (method == "BCp") {
          z0 <- stats::qnorm(mean(reps_clean < point_est, na.rm = TRUE))
          if (!is.finite(z0)) z0 <- 0
          z_alpha_lower <- stats::qnorm(a / 2)
          z_alpha_upper <- stats::qnorm(1 - a / 2)
          p_lower <- stats::pnorm(2 * z0 + z_alpha_lower)
          p_upper <- stats::pnorm(2 * z0 + z_alpha_upper)
          stats::quantile(reps_clean, probs = c(p_lower, p_upper), na.rm = TRUE)
        } else {
          b_type <- switch(method,
            "normal" = "norm",
            "basic" = "basic",
            "percentile" = "perc",
            "studentized" = "stud",
            "BCa" = "bca"
          )
          
          ci_res <- boot::boot.ci(boot_out, conf = conf, type = b_type, index = idx_num)
          switch(b_type,
            "norm" = ci_res$normal[2:3],
            "basic" = ci_res$basic[4:5],
            "perc" = ci_res$percent[4:5],
            "stud" = ci_res$student[4:5],
            "bca" = ci_res$bca[4:5]
          )
        }
      }, error = function(e) {
        stats::quantile(reps_clean, probs = c(a / 2, 1 - a / 2), na.rm = TRUE)
      })
      
      bounds[is.na(bounds) | !is.finite(bounds)] <- point_est
      
      ci_list[[length(ci_list) + 1]] <- data.frame(
        index = idx_name,
        estimate = point_est,
        method = method,
        type = type,
        alpha = a,
        conf_level = paste0(round((1 - a) * 100, 1), "%"),
        lower = bounds[1],
        upper = bounds[2],
        width = bounds[2] - bounds[1],
        stringsAsFactors = FALSE
      )
    }
  }
  
  ci_table <- do.call(rbind, ci_list)
  rownames(ci_table) <- NULL
  
  structure(
    list(
      ci_table = ci_table,
      boot_out = boot_out,
      fit = fit
    ),
    class = "gpci_ci_censor"
  )
}

#' @rdname boot_ci_censor
#' @export
gpci_boot_censor <- boot_ci_censor

#' Print Method for gpci_ci_censor
#'
#' @param x An object of class \code{gpci_ci_censor}.
#' @param ... Additional print arguments.
#' @export
print.gpci_ci_censor <- function(x, ...) {
  cat("--- Bootstrap Confidence Intervals (Interval-Censored Data) ---\n")
  cat("Bootstrap Type:   ", x$ci_table$type[1], "\n")
  cat("CI Method:        ", x$ci_table$method[1], "\n")
  cat("Replicates (B):   ", x$boot_out$R, "\n\n")
  
  print(round_df_censor(x$ci_table, 4))
  invisible(x)
}

#' Summary Method for gpci_ci_censor
#'
#' @param object An object of class \code{gpci_ci_censor}.
#' @param ... Additional arguments.
#' @export
summary.gpci_ci_censor <- function(object, ...) {
  print(object, ...)
  invisible(object)
}

#' Plot Method for gpcifit_censor Objects
#'
#' Visualizes interval-censored observations, fitted distribution density curve, and specification limits.
#'
#' @param x An object of class \code{gpcifit_censor}.
#' @param main Title of the plot.
#' @param ... Additional graphical parameters.
#' @export
plot.gpcifit_censor <- function(x, main = "Process Capability Analysis (Interval-Censored)", ...) {
  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar))
  
  graphics::par(mfrow = c(1, 2), mar = c(4, 4, 3, 1))
  
  # Plot 1: Interval observations
  n <- length(x$data_left)
  if (n > 0) {
    y_vals <- 1:n
    xlim_range <- range(c(x$data_left, x$data_right, x$LSL, x$USL), na.rm = TRUE)
    graphics::plot(x$data_left, y_vals, xlim = xlim_range, ylim = c(0, n + 1),
                   xlab = "Process Measurement", ylab = "Observation Index",
                   main = "Interval-Censored Data", pch = 16, col = "blue", ...)
    graphics::points(x$data_right, y_vals, pch = 17, col = "red")
    for (i in 1:n) {
      graphics::segments(x$data_left[i], i, x$data_right[i], i, col = "gray50", lty = 1)
    }
    graphics::abline(v = x$LSL, col = "red", lty = 2, lwd = 2)
    graphics::abline(v = x$USL, col = "red", lty = 2, lwd = 2)
    graphics::abline(v = x$target, col = "darkgreen", lty = 3, lwd = 2)
    graphics::legend("topleft", legend = c("Left Bound", "Right Bound", "Spec Limits", "Target"),
                     col = c("blue", "red", "red", "darkgreen"), pch = c(16, 17, NA, NA),
                     lty = c(NA, NA, 2, 3), cex = 0.8)
  } else {
    graphics::plot(1, 1, type = "n", main = "No Raw Data")
  }
  
  # Plot 2: Fitted PDF and Spec Limits
  grid_x <- seq(x$LSL - 0.5 * (x$USL - x$LSL), x$USL + 0.5 * (x$USL - x$LSL), length.out = 300)
  pdf_vals <- sapply(grid_x, function(v) {
    val <- tryCatch(do.call(x$distribution$pdf, c(list(v), x$distribution$params)), error = function(e) 0)
    if (is.na(val) || !is.finite(val)) 0 else val
  })
  
  graphics::plot(grid_x, pdf_vals, type = "l", col = "darkblue", lwd = 2,
                 xlab = "x", ylab = "Density", main = main, ...)
  graphics::abline(v = x$LSL, col = "red", lty = 2, lwd = 2)
  graphics::abline(v = x$USL, col = "red", lty = 2, lwd = 2)
  graphics::abline(v = x$target, col = "darkgreen", lty = 3, lwd = 2)
  
  invisible(x)
}

#' Plot Method for gpci_ci_censor Objects
#'
#' Visualizes bootstrap confidence intervals across different significance levels.
#'
#' @param x An object of class \code{gpci_ci_censor}.
#' @param ... Additional graphical parameters.
#' @export
plot.gpci_ci_censor <- function(x, ...) {
  tbl <- x$ci_table
  indices <- unique(tbl$index)
  n_idx <- length(indices)
  
  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar))
  
  graphics::par(mfrow = c(ceiling(n_idx / 2), min(2, n_idx)), mar = c(4, 4, 3, 1))
  
  for (idx in indices) {
    sub_tbl <- tbl[tbl$index == idx, ]
    xlim_val <- range(c(sub_tbl$lower, sub_tbl$upper, sub_tbl$estimate), na.rm = TRUE)
    if (diff(xlim_val) == 0) xlim_val <- xlim_val + c(-0.1, 0.1)
    
    y_pos <- seq_len(nrow(sub_tbl))
    graphics::plot(sub_tbl$estimate, y_pos, xlim = xlim_val, ylim = c(0, nrow(sub_tbl) + 1),
                   yaxt = "n", xlab = "Estimate & CI", ylab = "",
                   main = paste("Bootstrap CI:", idx), pch = 19, col = "blue", ...)
    graphics::axis(2, at = y_pos, labels = sub_tbl$conf_level, las = 1)
    
    for (i in seq_len(nrow(sub_tbl))) {
      graphics::segments(sub_tbl$lower[i], y_pos[i], sub_tbl$upper[i], y_pos[i], col = "darkblue", lwd = 2)
    }
  }
  
  invisible(x)
}

#' Plot Method for gpci_diagnostics_censor Objects
#'
#' Visualizes standard errors for parameters and capability indices.
#'
#' @param x An object of class \code{gpci_diagnostics_censor}.
#' @param ... Additional graphical parameters.
#' @export
plot.gpci_diagnostics_censor <- function(x, ...) {
  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar))
  
  graphics::par(mfrow = c(1, 2), mar = c(4, 5, 3, 1))
  
  # Parameter SEs
  se_p <- x$se_params
  graphics::barplot(se_p, main = "SE: Parameters", col = "skyblue", las = 2, ylab = "Standard Error", ...)
  
  # Index SEs
  se_i <- x$se_indices
  graphics::barplot(se_i, main = "SE: Capability Indices", col = "salmon", las = 2, ylab = "Standard Error", ...)
  
  invisible(x)
}

#' Autoplot Generic and Methods for Compatibility
#'
#' @param object Object to plot.
#' @param ... Additional arguments.
#' @export
autoplot <- function(object, ...) {
  UseMethod("autoplot")
}

#' @rdname autoplot
#' @export
autoplot.gpcifit_censor <- function(object, ...) {
  plot.gpcifit_censor(object, ...)
}

#' @rdname autoplot
#' @export
autoplot.gpci_ci_censor <- function(object, ...) {
  plot.gpci_ci_censor(object, ...)
}

#' @rdname autoplot
#' @export
autoplot.gpci_diagnostics_censor <- function(object, ...) {
  plot.gpci_diagnostics_censor(object, ...)
}

#' Fit Distribution Parameters under Interval-Censored Data
#'
#' Fits the parameters of a \code{gpci_dist_censor} distribution to interval-censored data
#' using Maximum Likelihood Estimation (MLE) via the MleCensoR package.
#'
#' @param data_left Numeric vector of left interval bounds for interval-censored data.
#' @param data_right Numeric vector of right interval bounds for interval-censored data.
#'   For exact observations, data_left = data_right.
#' @param dist A \code{gpci_dist_censor} distribution object with initial parameters.
#' @param start Optional named list of starting parameter values.
#' @return A new \code{gpci_dist_censor} object containing fitted parameters and variance-covariance attribute \code{"vcov"}.
#' @export
#' @examples
#' \dontrun{
#' dist_norm <- dist_normal()
#' left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
#' right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
#' fit <- fit_distribution_censor(left, right, dist_norm)
#' }
fit_distribution_censor <- function(data_left, data_right, dist, start = NULL) {
  if (!inherits(dist, "gpci_dist_censor")) {
    stop("dist must be of class 'gpci_dist_censor'")
  }
  
  if (length(data_left) != length(data_right)) {
    stop("data_left and data_right must have the same length")
  }
  
  if (is.null(start)) {
    start <- dist$params
  }
  
  if (length(start) == 0) {
    return(dist)
  }
  
  start_vec <- unlist(start)
  param_names <- names(start)
  
  # Prepare wrapper functions for MleCensoR::mle_interval
  pdf_wrapper <- function(x, theta) {
    names(theta) <- param_names
    par_list <- as.list(theta)
    vals <- tryCatch(do.call(dist$pdf, c(list(x), par_list)), error = function(e) rep(0, length(x)))
    vals[is.na(vals) | !is.finite(vals)] <- 0
    vals
  }
  
  cdf_wrapper <- function(x, theta) {
    names(theta) <- param_names
    par_list <- as.list(theta)
    vals <- tryCatch(do.call(dist$cdf, c(list(x), par_list)), error = function(e) rep(0, length(x)))
    vals[is.na(vals) | !is.finite(vals)] <- 0
    vals
  }
  
  surv_wrapper <- function(x, theta) {
    names(theta) <- param_names
    par_list <- as.list(theta)
    vals <- tryCatch(do.call(dist$sf, c(list(x), par_list)), error = function(e) rep(0, length(x)))
    vals[is.na(vals) | !is.finite(vals)] <- 0
    vals
  }
  
  # Primary estimation via MleCensoR::mle_interval
  mle_fit <- tryCatch({
    MleCensoR::mle_interval(
      l = data_left,
      r = data_right,
      pdf = pdf_wrapper,
      cdf = cdf_wrapper,
      surv = surv_wrapper,
      start = start_vec
    )
  }, error = function(e) {
    NULL
  })
  
  fitted_params <- NULL
  vcov_mat <- NULL
  conv_code <- 0
  
  if (!is.null(mle_fit) && !is.null(mle_fit$estimate) && all(is.finite(mle_fit$estimate))) {
    fitted_vec <- mle_fit$estimate
    names(fitted_vec) <- param_names
    fitted_params <- as.list(fitted_vec)
    
    if (!is.null(mle_fit$hessian) && is.matrix(mle_fit$hessian) && all(is.finite(mle_fit$hessian))) {
      vcov_mat <- tryCatch(solve(-mle_fit$hessian), error = function(e) NULL)
    }
  }
  
  # Fallback to internal robust MLE optimization if MleCensoR did not return valid estimates
  if (is.null(fitted_params)) {
    neg_log_lik <- function(par) {
      names(par) <- param_names
      par_list <- as.list(par)
      
      if (any(names(par) %in% c("sd", "scale", "shape", "shape1", "shape2", "sdlog") & par <= 0)) {
        return(1e10)
      }
      if (any(is.na(par) | !is.finite(par))) {
        return(1e10)
      }
      
      log_lik <- 0
      for (i in seq_along(data_left)) {
        L <- data_left[i]
        R <- data_right[i]
        
        if (is.na(L) || is.na(R) || !is.finite(L) || !is.finite(R)) return(1e10)
        
        if (L == R) {
          pdf_val <- tryCatch(do.call(dist$pdf, c(list(L), par_list)), error = function(e) 0)
          if (is.na(pdf_val) || !is.finite(pdf_val) || pdf_val <= 0) return(1e10)
          log_lik <- log_lik + log(max(pdf_val, 1e-300))
        } else {
          cdf_L <- tryCatch(do.call(dist$cdf, c(list(L), par_list)), error = function(e) 0)
          cdf_R <- tryCatch(do.call(dist$cdf, c(list(R), par_list)), error = function(e) 1)
          if (is.na(cdf_L) || !is.finite(cdf_L)) cdf_L <- 0
          if (is.na(cdf_R) || !is.finite(cdf_R)) cdf_R <- 1
          prob <- cdf_R - cdf_L
          if (is.na(prob) || !is.finite(prob) || prob <= 0) return(1e10)
          log_lik <- log_lik + log(max(prob, 1e-300))
        }
      }
      -log_lik
    }
    
    opt <- tryCatch({
      stats::optim(par = start_vec, fn = neg_log_lik, method = "Nelder-Mead", hessian = TRUE, control = list(maxit = 1000))
    }, error = function(e) {
      tryCatch({
        stats::optim(par = start_vec, fn = neg_log_lik, method = "BFGS", hessian = TRUE, control = list(maxit = 1000))
      }, error = function(e2) {
        res_nl <- stats::nlminb(start = start_vec, objective = neg_log_lik)
        hess <- tryCatch(numDeriv::hessian(neg_log_lik, res_nl$par), error = function(e3) diag(length(start_vec)))
        list(par = res_nl$par, hessian = hess, convergence = res_nl$convergence)
      })
    })
    
    fitted_params <- as.list(opt$par)
    names(fitted_params) <- param_names
    conv_code <- opt$convergence
    
    if (is.null(vcov_mat)) {
      vcov_mat <- tryCatch({
        solve(opt$hessian)
      }, error = function(e) {
        matrix(0, nrow = length(start), ncol = length(start), dimnames = list(param_names, param_names))
      })
    }
  }
  
  # Ensure fitted parameters are valid and positive where required
  for (pname in names(fitted_params)) {
    if (is.na(fitted_params[[pname]]) || !is.finite(fitted_params[[pname]])) {
      fitted_params[[pname]] <- start[[pname]]
    }
    if (pname %in% c("sd", "scale", "shape", "shape1", "shape2", "sdlog")) {
      fitted_params[[pname]] <- max(fitted_params[[pname]], 1e-6)
    }
  }
  
  if (is.null(vcov_mat) || any(is.na(vcov_mat)) || any(!is.finite(vcov_mat))) {
    vcov_mat <- matrix(0, nrow = length(start), ncol = length(start), dimnames = list(param_names, param_names))
  }
  
  res_dist <- define_distribution_censor(
    name = dist$name,
    pdf = dist$pdf,
    cdf = dist$cdf,
    sf = dist$sf,
    quantile = dist$quantile,
    params = fitted_params,
    support = dist$support
  )
  
  attr(res_dist, "vcov") <- vcov_mat
  attr(res_dist, "convergence") <- conv_code
  attr(res_dist, "data_left") <- data_left
  attr(res_dist, "data_right") <- data_right
  
  return(res_dist)
}

#' @rdname fit_distribution_censor
#' @export
gpci_fit_censor <- fit_distribution_censor

---
title: "Generalized Process Capability Indices for Interval-Censored Data"
author: "Shikhar Tyagi, Sumit Kumar, Arvind Pandey, Bhupendra Singh, Vrijesh Tripathi"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Generalized Process Capability Indices for Interval-Censored Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(gpciIntCensor)
```

## Introduction

The `gpciIntCensor` package provides a unified, comprehensive framework for evaluating Generalized Process Capability Indices (GPCIs) under interval-censored data using Maximum Likelihood Estimation (MLE) via `MleCensoR` and bootstrap confidence intervals.

Supported capability indices include:
* $C_{py}$ (Maiti et al., 2010)
* $S_{pmk}$ (Dey & Saha, 2019)
* $C_{pTk}$ (Saha et al., 2019)
* $C_{pc}$ (Saha et al., 2022)
* $C_{Npmc}$ (Alotaibi et al., 2022)
* $C_{Npmkc}$ (Saha et al., 2024)
* $C_{Npk}$ (Saha et al., 2018)
* Vännman's $C_p(u,v)$ family and quantile analogs.

## Workflow Example

### 1. Define Distribution and Generate Interval-Censored Data

```{r data_prep}
# Define normal distribution
dist_norm <- dist_normal(mean = 10, sd = 1.5)

# Simulate interval-censored data
set.seed(123)
true_vals <- rnorm(30, mean = 10, sd = 1.5)
data_left <- true_vals - 0.25
data_right <- true_vals + 0.25
```

### 2. Fit Parameters via MLE for Interval-Censored Data

```{r fit_dist}
dist_fitted <- fit_distribution_censor(data_left, data_right, dist_norm)
print(dist_fitted$params)
```

### 3. Compute Capability Indices

```{r capability_calc}
fit_cap <- capability_censor(
  data_left = data_left,
  data_right = data_right,
  distribution = dist_norm,
  USL = 14,
  LSL = 6,
  target = 10,
  indices = c("Cpy", "Cp", "Cpk", "Cpm", "Cpmk", "Spmk", "CpTk", "CNpmc"),
  mode = "moments"
)

print(fit_cap)
```

### 4. Bootstrap Confidence Intervals (90%, 95%, 99%)

```{r boot_ci_example}
ci_res <- boot_ci_censor(
  fit = fit_cap,
  B = 100,
  alpha = c(0.10, 0.05, 0.01),
  method = "percentile",
  type = "nonparametric"
)

print(ci_res)
```

### 5. Diagnostics: SE, MSE, and Coverage Probabilities

```{r diagnostics_example}
diag_res <- compute_diagnostics_censor(
  fit = fit_cap,
  true_params = list(mean = 10, sd = 1.5),
  true_indices = c(Cpy = 1.0, Cp = 1.33),
  B = 50
)

print(diag_res)
```

### 6. Visualization

```{r plot_example, fig.width=7, fig.height=4}
plot(fit_cap)
```

## References

* Maiti, S. S., Saha, M., & Nanda, A. K. (2010). On Generalizing Process Capability Indices. *Quality Technology & Quantitative Management*, 7(3), 279-300.
* Saha, M., Dey, S., & Maiti, S. S. (2018). Parametric and non-parametric bootstrap confidence intervals of CNpk for exponential power distribution. *Journal of Industrial and Production Engineering*, 35(3), 160-169.
* Dey, S., & Saha, M. (2019). Assessing the process capability index Spmk using improved estimators. *Life Cycle Reliability and Safety Engineering*, 8(3), 253-264.
* Saha, M., Dey, S., & Maiti, S. S. (2019). Bootstrap confidence intervals of CpTk for two parameter logistic exponential distribution with applications. *International Journal of System Assurance Engineering and Management*, 10(4), 861-872.
* Alotaibi, R., Dey, S., & Saha, M. (2022). Estimation and Confidence Intervals of a New PCI CNpmc for Logistic-Exponential Process Distribution. *Journal of Mathematics*, 2022, 3135264.
* Saha, M., Dey, S., & Nadarajah, S. (2022). Parametric inference of the process capability index Cpc for exponentiated exponential distribution. *Journal of Applied Statistics*, 49(16), 4097-4121.
* Saha, M., Tripathi, H., & Dey, S. (2024). Classical Inference of a New PCI CNpmkc for Logistic-Exponential Process Distribution. *International Journal of Reliability, Quality and Safety Engineering*, 31(3), 2450013.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.gpcifit_censor}
\alias{plot.gpcifit_censor}
\title{Plot Method for gpcifit_censor Objects}
\usage{
\method{plot}{gpcifit_censor}(x, main = "Process Capability Analysis (Interval-Censored)", ...)
}
\arguments{
\item{x}{An object of class \code{gpcifit_censor}.}

\item{main}{Title of the plot.}

\item{...}{Additional graphical parameters.}
}
\value{
Invisible fit object \code{x}.
}
\description{
Visualizes interval-censored observations, fitted distribution density curve, and specification limits.
}
\examples{
dist_norm <- dist_normal()
left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
fit <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
plot(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{boot_ci_censor}
\alias{boot_ci_censor}
\alias{gpci_boot_censor}
\alias{print.gpci_ci_censor}
\alias{summary.gpci_ci_censor}
\title{Compute Bootstrap Confidence Intervals for Process Capability Indices (Interval-Censored Data)}
\usage{
boot_ci_censor(
  fit,
  B = 2000,
  alpha = c(0.1, 0.05, 0.01),
  method = c("percentile", "normal", "basic", "BCa", "BCp", "studentized"),
  type = c("parametric", "nonparametric"),
  parallel = FALSE,
  ncpus = 1
)

gpci_boot_censor(
  fit,
  B = 2000,
  alpha = c(0.1, 0.05, 0.01),
  method = c("percentile", "normal", "basic", "BCa", "BCp", "studentized"),
  type = c("parametric", "nonparametric"),
  parallel = FALSE,
  ncpus = 1
)

\method{print}{gpci_ci_censor}(x, ...)

\method{summary}{gpci_ci_censor}(object, ...)
}
\arguments{
\item{fit}{A \code{gpcifit_censor} object returned by \code{\link{capability_censor}}.}

\item{B}{Number of bootstrap replicates.}

\item{alpha}{Vector of significance levels (default is 0.10, 0.05, 0.01).}

\item{method}{Confidence interval method ("percentile", "normal", "basic", "BCa", "BCp", "studentized").}

\item{type}{Bootstrap type ("parametric" or "nonparametric").}

\item{parallel}{Logical. If TRUE, parallel processing is used.}

\item{ncpus}{Number of CPUs for parallel execution.}

\item{x, object}{An object of class \code{gpci_ci_censor}.}

\item{...}{Additional print/summary arguments.}
}
\value{
An object of class \code{gpci_ci_censor}.
}
\description{
Runs parametric or non-parametric bootstrapping to compute confidence intervals
for generalized process capability indices at multiple significance levels (90\%, 95\%, 99\%)
under interval-censored data.
}
\examples{
\donttest{
dist_norm <- dist_normal()
left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
fit <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
ci <- boot_ci_censor(fit, B = 100, alpha = c(0.10, 0.05, 0.01), method = "percentile")
print(ci)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimation.R
\name{fit_distribution_censor}
\alias{fit_distribution_censor}
\alias{gpci_fit_censor}
\title{Fit Distribution Parameters under Interval-Censored Data}
\usage{
fit_distribution_censor(data_left, data_right, dist, start = NULL)

gpci_fit_censor(data_left, data_right, dist, start = NULL)
}
\arguments{
\item{data_left}{Numeric vector of left interval bounds for interval-censored data.}

\item{data_right}{Numeric vector of right interval bounds for interval-censored data.}

\item{dist}{A \code{gpci_dist_censor} distribution object with initial parameters.}

\item{start}{Optional named list of starting parameter values.}
}
\value{
A new \code{gpci_dist_censor} object containing fitted parameters.
}
\description{
Fits the parameters of a \code{gpci_dist_censor} distribution to interval-censored data
using Maximum Likelihood Estimation (MLE) via the MleCensoR package.
}
\examples{
dist_norm <- dist_normal()
left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
fit <- fit_distribution_censor(left, right, dist_norm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{autoplot}
\alias{autoplot}
\alias{autoplot.gpcifit_censor}
\alias{autoplot.gpci_ci_censor}
\alias{autoplot.gpci_diagnostics_censor}
\title{Autoplot Generic and Methods for Compatibility}
\usage{
autoplot(object, ...)

\method{autoplot}{gpcifit_censor}(object, ...)

\method{autoplot}{gpci_ci_censor}(object, ...)

\method{autoplot}{gpci_diagnostics_censor}(object, ...)
}
\arguments{
\item{object}{Object to plot.}

\item{...}{Additional arguments.}
}
\value{
Invisible plot object.
}
\description{
Autoplot Generic and Methods for Compatibility
}
\examples{
dist_norm <- dist_normal()
left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
fit <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
autoplot(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.gpci_ci_censor}
\alias{plot.gpci_ci_censor}
\title{Plot Method for gpci_ci_censor Objects}
\usage{
\method{plot}{gpci_ci_censor}(x, ...)
}
\arguments{
\item{x}{An object of class \code{gpci_ci_censor}.}

\item{...}{Additional graphical parameters.}
}
\value{
Invisible CI object \code{x}.
}
\description{
Visualizes bootstrap confidence intervals across different significance levels.
}
\examples{
\donttest{
dist_norm <- dist_normal()
left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
fit <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
ci <- boot_ci_censor(fit, B = 100, alpha = c(0.10, 0.05, 0.01), method = "percentile")
plot(ci)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indices.R
\name{capability_censor}
\alias{capability_censor}
\alias{print.gpcifit_censor}
\alias{summary.gpcifit_censor}
\alias{coef.gpcifit_censor}
\alias{vcov.gpcifit_censor}
\alias{confint.gpcifit_censor}
\title{Compute Process Capability Indices for Interval-Censored Data}
\usage{
capability_censor(
  data_left = NULL,
  data_right = NULL,
  distribution,
  USL,
  LSL,
  target = (USL + LSL)/2,
  indices = c("Cpy", "Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk"),
  u = 1,
  v = 1,
  mode = c("moments", "quantile"),
  fit = TRUE,
  C0 = 1,
  C1 = 0,
  C2 = 1,
  tolerance_t = USL - LSL,
  P0 = 0.9973002,
  LDL = LSL,
  UDL = USL
)

\method{print}{gpcifit_censor}(x, ...)

\method{summary}{gpcifit_censor}(object, ...)

\method{coef}{gpcifit_censor}(object, what = c("indices", "parameters"), ...)

\method{vcov}{gpcifit_censor}(object, ...)

\method{confint}{gpcifit_censor}(
  object,
  parm = NULL,
  level = 0.95,
  B = 1000,
  method = "percentile",
  ...
)
}
\arguments{
\item{data_left}{Numeric vector of left interval bounds for interval-censored data.}

\item{data_right}{Numeric vector of right interval bounds for interval-censored data.}

\item{distribution}{A \code{gpci_dist_censor} distribution object.}

\item{USL}{Numeric value of the Upper Specification Limit.}

\item{LSL}{Numeric value of the Lower Specification Limit.}

\item{target}{Numeric value of process target (default is midpoint of USL and LSL).}

\item{indices}{Character vector of capability indices to compute.}

\item{u, v}{Parameters for Vannman's Cp(u,v) family.}

\item{mode}{Computation mode: "moments" or "quantile".}

\item{fit}{Logical. If TRUE, parameters are estimated via MLE under interval censoring.}

\item{C0, C1, C2}{Cost function parameters for CNpmc and CNpmkc.}

\item{tolerance_t}{Tolerance parameter for CNpmc and CNpmkc.}

\item{P0}{Desirable process yield for Cpy and Cpc.}

\item{LDL, UDL}{Desired limits for CpTk.}

\item{x, object}{An object of class \code{gpcifit_censor}.}

\item{what}{Extraction target: "indices" or "parameters".}

\item{parm}{Optional vector of index names.}

\item{level}{Confidence level(s).}

\item{B}{Number of bootstrap replicates.}

\item{method}{Bootstrap CI method.}

\item{...}{Additional arguments.}
}
\value{
An object of class \code{gpcifit_censor}.
}
\description{
Computes classical and generalized Process Capability Indices (PCIs) for interval-censored data.
}
\examples{
dist_norm <- dist_normal()
left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
capability_censor(
  data_left = left, data_right = right,
  distribution = dist_norm,
  USL = 13, LSL = 7, target = 10,
  indices = c("Cpy", "Cp", "Cpk", "Cpm", "Cpmk"),
  mode = "moments"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{define_distribution_censor}
\alias{define_distribution_censor}
\alias{gpci_dist_censor}
\title{Define a Process Distribution for Interval-Censored Data}
\usage{
define_distribution_censor(
  name,
  pdf = NULL,
  cdf = NULL,
  sf = NULL,
  quantile = NULL,
  params = list(),
  support = c(-Inf, Inf)
)

gpci_dist_censor(
  name,
  pdf = NULL,
  cdf = NULL,
  sf = NULL,
  quantile = NULL,
  params = list(),
  support = c(-Inf, Inf)
)
}
\arguments{
\item{name}{Character string naming the distribution.}

\item{pdf}{Function representing the probability density function (PDF/PMF).}

\item{cdf}{Function representing the cumulative distribution function (CDF).}

\item{sf}{Function representing the survival function (SF = 1 - CDF).}

\item{quantile}{Function representing the quantile function.}

\item{params}{Named list of parameters for the distribution.}

\item{support}{Vector of length 2 defining lower and upper bounds of support.}
}
\value{
An object of class \code{gpci_dist_censor}.
}
\description{
Constructor to define a probability distribution for process capability analysis
under interval-censored data.
}
\examples{
custom_weib <- define_distribution_censor(
  name = "custom_weibull",
  sf = function(x, shape, scale) pweibull(x, shape, scale, lower.tail = FALSE),
  params = list(shape = 2, scale = 10),
  support = c(0, Inf)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{compute_diagnostics_censor}
\alias{compute_diagnostics_censor}
\alias{gpci_diagnostics_censor}
\alias{print.gpci_diagnostics_censor}
\alias{summary.gpci_diagnostics_censor}
\title{Compute Standard Errors, MSE, and Coverage Probabilities for Interval-Censored Data}
\usage{
compute_diagnostics_censor(
  fit,
  true_params = NULL,
  true_indices = NULL,
  B = 1000,
  alpha = c(0.1, 0.05, 0.01)
)

gpci_diagnostics_censor(
  fit,
  true_params = NULL,
  true_indices = NULL,
  B = 1000,
  alpha = c(0.1, 0.05, 0.01)
)

\method{print}{gpci_diagnostics_censor}(x, ...)

\method{summary}{gpci_diagnostics_censor}(object, ...)
}
\arguments{
\item{fit}{A \code{gpcifit_censor} object returned by \code{\link{capability_censor}}.}

\item{true_params}{Named list of true parameter values (if known).}

\item{true_indices}{Named vector of true index values (if known).}

\item{B}{Number of bootstrap replicates for SE computation.}

\item{alpha}{Vector of significance levels for coverage probability.}

\item{x, object}{An object of class \code{gpci_diagnostics_censor}.}

\item{...}{Additional print/summary arguments.}
}
\value{
A list of class \code{gpci_diagnostics_censor}.
}
\description{
Computes standard errors, mean squared errors, and coverage probabilities for
distribution parameters and process capability indices using bootstrap results.
}
\examples{
\donttest{
dist_norm <- dist_normal()
left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
fit <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
diagnostics <- compute_diagnostics_censor(fit, B = 100)
print(diagnostics)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indices.R
\name{gpci_index}
\alias{gpci_index}
\title{Dispatcher for Single Index Computation under Interval Censoring}
\usage{
gpci_index(fit, index)
}
\arguments{
\item{fit}{A \code{gpcifit_censor} object.}

\item{index}{Name of index to compute.}
}
\value{
Numeric value of the specified index.
}
\description{
Dispatcher for Single Index Computation under Interval Censoring
}
\examples{
dist_norm <- dist_normal()
left <- c(9.5, 10.2, 8.8, 11.1, 9.9)
right <- c(10.5, 11.2, 9.8, 12.1, 10.9)
fit <- capability_censor(left, right, dist_norm, USL = 13, LSL = 7)
gpci_index(fit, "Cpy")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{builtin_dist}
\alias{builtin_dist}
\alias{dist_normal}
\alias{dist_lognormal}
\alias{dist_gamma}
\alias{dist_weibull}
\alias{dist_beta}
\alias{dist_logistic_exponential}
\title{Built-in Process Distributions for Interval-Censored Analysis}
\usage{
dist_normal(mean = 0, sd = 1)

dist_lognormal(meanlog = 0, sdlog = 1)

dist_gamma(shape = 1, scale = 1)

dist_weibull(shape = 1, scale = 1)

dist_beta(shape1 = 1, shape2 = 1)

dist_logistic_exponential(shape = 1, scale = 1)
}
\arguments{
\item{mean, sd}{Mean and standard deviation parameters for normal distribution.}

\item{meanlog, sdlog}{Mean and standard deviation of the distribution on the log scale.}

\item{shape, scale}{Shape and scale parameters.}

\item{shape1, shape2}{Non-negative parameters of the Beta distribution.}
}
\value{
An object of class \code{gpci_dist_censor}.
}
\description{
Convenience constructors for standard process probability distributions.
}
\examples{
dist_n <- dist_normal(mean = 10, sd = 2)
dist_le <- dist_logistic_exponential(shape = 1.5, scale = 0.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{compute_theoretical_moments_censor}
\alias{compute_theoretical_moments_censor}
\title{Compute Theoretical Moments of an Interval-Censored Distribution}
\usage{
compute_theoretical_moments_censor(dist)
}
\arguments{
\item{dist}{A \code{gpci_dist_censor} object.}
}
\value{
A list with \code{mean} and \code{var}.
}
\description{
Computes the theoretical mean and variance of a \code{gpci_dist_censor} object using numerical integration.
}
\examples{
dist_n <- dist_normal(mean = 5, sd = 1.5)
compute_theoretical_moments_censor(dist_n)
}
